@@ -3,7 +3,7 @@ from pyg_base._as_list import as_list, as_tuple
 from pyg_base._as_float import as_float
 from pyg_base._dict import Dict
 from pyg_base._zip import zipper, lens
-from pyg_base._types import is_str, is_strs, is_arr, is_df, is_dicts, is_int, is_ints, is_tuple, is_bools, is_nan
+from pyg_base._types import is_str, is_strs, is_arr, is_df, is_dicts, is_int, is_ints, is_tuple, is_bools, is_nan, is_num
 from pyg_base._tree import is_tree, tree_to_table
 from pyg_base._inspect import getargs
 from pyg_base._decorators import kwargs_support, try_none, try_back
@@ -35,7 +35,12 @@ def relabel_lower(v):
         return v.strip().replace(' ', '_').replace('"','').replace("'",'').replace(',','').lower()
     else:
         return v    
-    
+
+
+def _dict_in_place_update(a, b):
+    a.update(b)
+    return a
+
 def dict_concat(*dicts):
     """
     A method of turning a list of dicts into one happy dict sharing its keys
@@ -594,8 +599,9 @@ class dictable(Dict):
 
     
     def apply(self, function, **default_params):
-        f = kwargs_support(function)
-        return [f(**(default_params|row)) for row in self]
+        f = kwargs_support(function)        
+        return [f(**_dict_in_place_update(default_params,row)) for row in self] ## we update in place since all rows share same keys
+
 
     def do(self, function, *keys):
         """
@@ -785,7 +791,7 @@ class dictable(Dict):
         return type(self)({key: [value[i] for i in rows] for key, value in self.items()})
 
     def __add__(self, other):
-        if other in (None, 0):
+        if other is None or (is_num(other) and other == 0):
             return self
         return self.concat(self, other)
 
